@@ -1,7 +1,17 @@
+# burgers.py
+# Written by Thomas Hilder and Francesco Bollati
+
+"""
+Contains the solve_burgers function used in the wake non-linear propagation calculations.
+"""
+
 import numpy                as np
 import matplotlib.pyplot    as plt
 
-def solve_burgers(
+# NOTE: contents are intended for internal use and should not be directly accessed by users
+
+# function to solve burgers equation as part of the non-linear propagation of the planet wake
+def _solve_burgers(
     eta, 
     profile, 
     gamma, 
@@ -15,12 +25,11 @@ def solve_burgers(
     show_teta, 
     tf_fac
 ): 
-    """ Solve eq. (10) Bollati et al. 2021
+    """Propagate the wake in (t,eta,chi) space by solving Eq. 10 from Bollati et al. 2021 using Godunov scheme.
     """
 
     # Eq. (15) Bollati et al. 2021
     profile = profile * (gamma + 1) * beta_p / 2**(3 / 4)
-
     deta    = eta[1] - eta[0]
 
     # time required to develop N-wave for betap = 1
@@ -76,48 +85,11 @@ def solve_burgers(
     # define flux vector
     F = np.zeros(Neta + 1) #there is a flux at each cell edge
 
-    # define the flux function of Burgers equation
-    def flux(u):
-        return 0.5 * u**2
-
-    # define the Central difference numerical flux---> ritorna la media aritmetica dei flux sx e dx passati
-    """
-    def CentralDifferenceFlux(uL,uR):
-        # compute physical fluxes at left and right state
-        FL = flux(uL)
-        FR = flux(uR)
-        return 0.5*(FL+FR)
+    # calculate flux according to Godunov scheme, vectorised
+    def _NumericalFluxVector(uL, uR):
+        """Calculate flux according to the Godunov scheme.
         """
 
-    def GodunovNumericalFlux(uL, uR):
-
-        # compute physical fluxes at left and right state
-        FL = flux(uL)
-        FR = flux(uR)
-
-        # compute the shock speed
-        s = 0.5 * (uL + uR)
-
-        # from Toro's book
-        if (uL >= uR):
-            if (s > 0.0):
-                return FL
-            else:
-                return FR
-        else:
-            if (uL > 0.0):
-                return FL
-            elif (uR < 0.0):
-                return FR
-            else:
-                return 0.0
-
-    def NumericalFlux(uL, uR):
-        # return CentralDifferenceFlux(uL,uR)
-        return GodunovNumericalFlux(uL, uR)
-
-
-    def NumericalFluxVector(uL, uR):
         FL = 0.5 * uL**2
         FR = 0.5 * uR**2
 
@@ -152,7 +124,7 @@ def solve_burgers(
         lapsed_time += dt
 
         # compute the interior fluxes
-        F[1:Neta] = NumericalFluxVector(solution[-1][0:-1], solution[-1][1:])
+        F[1:Neta] = _NumericalFluxVector(solution[-1][0:-1], solution[-1][1:])
 
         # compute the left boundary flux
         if solution[-1][0] < 0.0:
@@ -161,7 +133,7 @@ def solve_burgers(
             uL = solution[0][0]
 
         uR   = solution[-1][0]
-        F[0] = NumericalFlux(uL, uR)
+        F[0] = _NumericalFluxVector(uL, uR)
 
         # compute the right boundary flux
         if solution[-1][Neta - 1] > 0.0:
@@ -170,7 +142,7 @@ def solve_burgers(
             uR = solution[0][Neta - 1]
 
         uL      = solution[-1][Neta - 1]
-        F[Neta] = NumericalFlux(uL, uR)
+        F[Neta] = _NumericalFluxVector(uL, uR)
 
         solution.append(solution[-1][0:Neta] - dt / deta * (F[1:Neta+1] - F[0:Neta]))
 
@@ -178,19 +150,17 @@ def solve_burgers(
     time     = np.array(time)
 
     # plot Fig. 3 Bollati et al. 2021
-    r"""
-    plt.plot(eta, solution[:,0], label = "$t=t_0+$ ")#+str(0*dt))
-    plt.plot(eta, solution[:,100], label = "$t=t_0+$ ")#+str(round(dt*Nt/10,2)))
-    plt.plot(eta, solution[:,200], label = "$t=t_0+$ ")#+str(round(dt*Nt/6,2)))
-    plt.plot(eta, solution[:,300], label = "$t=t_0+$ ")#+str(round(dt*Nt/3,2)))
-    plt.plot(eta, solution[:,400], label = "$t=t_0+$ ")#+str(round(T,2)))
-    plt.legend()
-    plt.xlabel(r"$\eta$")
-    plt.ylabel(r"$\chi(t,\eta)$")
-    plt.grid(True)
-    plt.title(r'$\chi$ "evolution" $r > r_p$')
-    plt.show()
-    """
+    #plt.plot(eta, solution[:,0], label = "$t=t_0+$ ")#+str(0*dt))
+    #plt.plot(eta, solution[:,100], label = "$t=t_0+$ ")#+str(round(dt*Nt/10,2)))
+    #plt.plot(eta, solution[:,200], label = "$t=t_0+$ ")#+str(round(dt*Nt/6,2)))
+    #plt.plot(eta, solution[:,300], label = "$t=t_0+$ ")#+str(round(dt*Nt/3,2)))
+    #plt.plot(eta, solution[:,400], label = "$t=t_0+$ ")#+str(round(T,2)))
+    #plt.legend()
+    #plt.xlabel(r"$\eta$")
+    #plt.ylabel(r"$\chi(t,\eta)$")
+    #plt.grid(True)
+    #plt.title(r'$\chi$ "evolution" $r > r_p$')
+    #plt.show()
 
     if show_teta: # combining linear and non-linear solution and plotting
 
@@ -211,14 +181,6 @@ def solve_burgers(
         ax.set_ylabel('$\eta$')
         #plt.savefig("teta_badjoin.pdf")
         plt.show()
-    
-    #Nt = np.shape(solution)[1]
-
-    #solution_inner = np.zeros(solution.shape) # solution for r < Rp (and disc rotating counterclockwise)
-    #eta_inner = - eta[::-1]
-    #for i in range(Neta):
-    #    for j in range(Nt):
-    #        solution_inner[i,j] = solution[int(Neta-1-i),j]
 
     # The following will put the linear solution into returned solution for Chi
     if False:
