@@ -1,15 +1,83 @@
+# grid.py
+# Written by Thomas Hilder
+
+"""
+Contains the Grid class on which all Wakeflow models are run/stored.
+"""
+
 import numpy                as np
 import matplotlib.pyplot    as plt
 import astropy.io.fits      as fits
 import shutil               as sh
 from scipy.interpolate  import RectBivariateSpline
 from matplotlib.colors  import LogNorm
-from .mcfost_interface  import read_mcfost_grid_data
+from .mcfost_interface  import _read_mcfost_grid_data
+from .model_setup       import _Parameters
+from .linear_perts      import _LinearPerts
+from .non_linear_perts  import _NonLinearPerts
 
-# NOTE: grid dimensions are (x,z,y) or (phi,z,r)
+# NOTE: contents are intended mostly for internal use and should not really be accessed by users, but I have still provided
+# documentation in the case that advanced users want to mess around with any of it.
 
-class Grid:
-    def __init__(self, parameters):
+# a class to store numpy arrays of results together, so that nothing gets confused. Also stores information and parameters
+class _Grid:
+    """
+    Grid object to generate and store Wakeflow results on. 
+
+    Note that all 3D arrays in the Grid have dimensions (x,z,y) or (phi,z,r).
+
+    Attributes
+    ----------
+    p : Parameters
+        Parameters object (model_setup.py) containing the options used for the model.
+    info : dict
+        Contains the type, size and contents of the grid.
+    height: ndarray
+        Scale height of the disk.
+    v_r : ndarray
+        Contains radial velocities.
+    v_phi : ndarray
+        Contains azimuthal velocities, positive is counterclockwise.
+    rho : ndarray
+        Contains densities.
+    x : ndarray
+        1D. Contains x coordinates of Cartesian grid.
+    y : ndarray
+        1D. Contains y coordinates of Cartesian grid.
+    z_xy : ndarray
+        1D. Contains z coordinates of Cartesian grid.
+    X : ndarray
+        3D. Contains x coordinates of Cartesian grid points.
+    Y : ndarray
+        3D. Contains y coordinates of Cartesian grid points.
+    Z_xy : ndarray
+        3D. Contains z coordinates of Cartesian grid points.
+    R_xy : ndarray
+        3D. Contains radial coordinates of Cartesian grid points.
+    PHI_xy : ndarray
+        3D. Contains azimuthal coordinates of Cartesian grid points.
+    r : ndarray
+        1D. Contains radial coordinates of Cylindrical or Mcfost grid.
+    phi : ndarray
+        1D. Contains azimuthal coordinates of Cylindrical or Mcfost grid.
+    z : ndarray
+        1D. Contains z coordinates of Cylindrical grid.
+    R : ndarray
+        3D. Contains radial coordinates of Cylindrical or Mcfost grid points.
+    PHI : ndarray
+        3D. Contains azimuthal coordinates of Cylindrical or Mcfost grid points.
+    Z : ndarray
+        3D. Contains z coordinates of Cylindrical or Mcfost grid points.
+    """
+
+    # instantiate the class
+    def __init__(self, parameters : _Parameters) -> None:
+        """
+        Parameters
+        ----------
+        parameters : _Parameters
+            _Parameters object (model_setup.py) containing parameters to be used for the Grid.
+        """
 
         # grab parameters object
         self.p = parameters
@@ -27,28 +95,32 @@ class Grid:
         self.v_phi = None 
         self.rho   = None
 
-    def make_grid(self):
-
-        #print(f"Constructing {self.p.grid_type} Grid ")
+    # setup the grids with chosen geometry
+    def _make_grid(self) -> None:
+        """Generates grid according to parameters in self.p and define disk scale height
+        """
 
         # define disk height (not used for mcfost grid)
         self.height = self.p.hr * (self.p.r_outer / self.p.r_ref)**(0.5 - self.p.q) * self.p.r_outer
 
         # make cartesian grid
         if self.p.grid_type == "cartesian":
-            self.make_cartesian_grid()
+            self._make_cartesian_grid()
 
         # make cylindrical grid
         elif self.p.grid_type == "cylindrical":
-            self.make_cylindrical_grid()
+            self._make_cylindrical_grid()
 
         # make mcfost grid
         elif self.p.grid_type == "mcfost":
-            self.make_mcfost_grid()
+            self._make_mcfost_grid()
 
-    def make_cartesian_grid(self):
+    # setup Cartesian grid
+    def _make_cartesian_grid(self) -> None:
+        """Generates a Cartesian grid using parameters in self.p
+        """
         
-        # make grid from specifications in parameter file
+        # make grid from specified parameters
         self.x    = np.linspace(-self.p.r_outer, self.p.r_outer, self.p.n_x)
         self.y    = np.linspace(-self.p.r_outer, self.p.r_outer, self.p.n_y)
         self.z_xy = np.linspace(0, self.height, self.p.n_z)
@@ -61,7 +133,10 @@ class Grid:
         self.info["Size"][1] = self.z_xy.shape[0]
         self.info["Size"][2] = self.y.shape   [0]
 
-    def make_cylindrical_grid(self):
+    # setup Cylindrical grid
+    def _make_cylindrical_grid(self) -> None:
+        """Generates a Cylindrical grid using parameters in self.p
+        """
          
         # make grid from specifications in parameter file
         if self.p.r_log:
@@ -81,10 +156,13 @@ class Grid:
         self.info["Size"][2] = self.r.shape  [0]
         self.info["Log_r"]   = self.p.r_log
 
-    def make_mcfost_grid(self):
+    # setup Mcfost grid by calling Mcfost to generate the geometry, and then read in that geometry
+    def _make_mcfost_grid(self) -> None:
+        """Generates an Mcfost grid using parameters in self.p
+        """
 
         # generate mcfost grid data from specifications in parameter file
-        self.R, self.Z = read_mcfost_grid_data(self.p)
+        self.R, self.Z = _read_mcfost_grid_data(self.p)
         self.r         = self.R[0, 0, :]
         self.phi       = np.linspace(0, 2*np.pi, self.p.n_phi)
 
@@ -100,14 +178,28 @@ class Grid:
         self.info["Size"][2] = self.r.shape  [0]
         self.info["Log_r"]   = "mcfost"
 
-    def get_r_phi_coords(self):
+    # we need (r,phi) points for the Cartesian grid points in order to do transformations
+    def _get_r_phi_coords(self) -> None:
+        """Find the (r,phi) coordinates corresponding to each (x,y) point
+        """
+
+        if self.info["Type"] != "cartesian":
+            raise Exception("Can only be used on Cartesian grid.")
 
         # need the equivalent (r,phi) coordinates when using cartesian grid
         self.R_xy   = np.sqrt   (self.X**2 + self.Y**2)
         self.PHI_xy = np.arctan2(self.Y,     self.X)
 
-    def remove_dimensions(self, scale_dens=False):
-        "This should only be used once all other steps are DONE, except for saving"
+    # if the user desires dimensionless results, needs to be used last after all grids are merged
+    def _remove_dimensions(self, scale_dens : bool = False) -> None:
+        """Convert results to be dimensionless by appropriate scaling; should be used last
+
+        Parameters
+        ----------
+        scale_dens : bool, optional
+            If true, the density is scaled by the rho_ref value in the parameters. Should be left as default for
+            scaling perturbations (delta rho / rho) and set to True otherwise.
+        """
 
         grid_length = self.p.U_len / self.p.au
 
@@ -140,8 +232,11 @@ class Grid:
         if scale_dens:
             self.rho /= self.p.rho_ref
 
-    def flip_results(self):
-        """Go from counterclockwise to clockwise rotation. Should only be used once all other steps are DONE, except for saving"""
+    # results for clockwise rotation are the same under a reflection about the y-axis, or phi-axis, 
+    # as well as a reverse of sign in v_phi
+    def _flip_results(self) -> None:
+        """Go from counterclockwise to clockwise rotation; should be used last
+        """
 
         if self.p.grid_type == "cartesian":
             self.y *= -1
@@ -153,20 +248,22 @@ class Grid:
 
         self.v_phi *= -1
 
-    def make_keplerian_disk(self):
-
-        #print("Making Keplerian disk ")
+    # setup the unperturbed background disk onto which the perturbations will be mapped
+    def _make_keplerian_disk(self) -> None:
+        """Generate unperturbed accretion disk in vertical hydrostatic equilibrium, according to parameters in self.p
+        """
 
         # get radii
         if self.info["Type"] == "cartesian":
-            self.get_r_phi_coords()
+            self._get_r_phi_coords()
             r = self.R_xy
             z = self.Z_xy
         else:
             r = self.R
             z = self.Z
 
-        p       = self.p.p
+        # rename for readability in long equations
+        p       = self.p.dens_p
         q       = self.p.q
         hr      = self.p.hr
         r_ref   = self.p.r_ref
@@ -180,12 +277,15 @@ class Grid:
 
         # Keplerian velocities
         self.v_r   = np.zeros((self.info["Size"][0], self.info["Size"][1], self.info["Size"][2]))
-        self.v_kep = 1e-5 * np.sqrt(self.p.G_const * self.p.m_star * self.p.m_solar / (r * self.p.au))
+        self.v_kep = 1e-5 * np.sqrt(G * m_star * m_sol / (r * au))
         self.v_phi = np.copy(self.v_kep) * self.p.a_cw
 
+        # get h/r(r)
+        hrf = hr * (r / r_ref) ** (0.5 - q)
+
         # pressure and gravity height dependent correction
         corr = np.sqrt(
-            -1* (p + q) * hr**2 * (r / r_ref)**(1 - 2*q) + (1 - q) + q*r / np.sqrt(r**2 + z**2)
+            (-1* (p + 2 * q) * hrf**2) + (1 - 2 * q) + (2 * q * r / np.sqrt(r**2 + z**2))
         )
 
         # perform correction
@@ -214,7 +314,10 @@ class Grid:
         # update grid info
         self.info["Contains"] = "Keplerian velocity field, unperturbed density"
 
-    def make_empty_disk(self):
+    # fill the grids with zeros, used for storing perturbations on separate grids before mapping onto background
+    def _make_empty_disk(self) -> None:
+        """fill v_r, v_phi and rho with zeros
+        """
 
         # make grids full of zeros
         self.v_r   = np.zeros((self.info["Size"][0], self.info["Size"][1], self.info["Size"][2]))
@@ -224,7 +327,17 @@ class Grid:
         # update grid info
         self.info["Contains"] = "Zeros"
 
-    def add_linear_perturbations(self, LinearPerts, rho_background):
+    # add results from the linear regime nearby the planet onto the grid
+    def _add_linear_perturbations(self, LinearPerts : _LinearPerts, rho_background : "_Grid.rho") -> None:
+        """Add results from the linear regime, stored in LinearPerts object, to the grid
+
+        Parameters
+        ----------
+        LinearPerts : LinearPerts
+            LinearPerts object containing results from the linear regime.
+        rho_background : Grid.rho
+            unperturbed density from Grid object where make_keplerian_disk has been used.
+        """
 
         # box size (in units of Hill radius), note for conversions that self.p.l = 1 Hill radius in cgs
         x_box_size = 2 * self.p.scale_box
@@ -241,7 +354,7 @@ class Grid:
 
         # find (phi, r) grid for either Cartesian or Cylindrical global grid
         if self.info["Type"] == "cartesian":
-            self.get_r_phi_coords()
+            self._get_r_phi_coords()
             R, PHI = self.R_xy, self.PHI_xy
         else:
             R, PHI = self.R, self.PHI
@@ -299,18 +412,26 @@ class Grid:
         self.rho += global_lin_rho[:, np.newaxis, :] * rho_background
 
         # plot for debugging
-        """
-        _, ax = plt.subplots(subplot_kw=dict(projection='polar'))
-        myplot = ax.contourf(PHI[:,0,:], R[:,0,:], global_lin_v_r, levels=300, cmap='RdBu')
-        ax.set_ylim(0, self.p.r_outer)
-        plt.colorbar(myplot)
-        plt.show()
-        """
+        #_, ax = plt.subplots(subplot_kw=dict(projection='polar'))
+        #myplot = ax.contourf(PHI[:,0,:], R[:,0,:], global_lin_v_r, levels=300, cmap='RdBu')
+        #ax.set_ylim(0, self.p.r_outer)
+        #plt.colorbar(myplot)
+        #plt.show()
 
         # update grid info
         self.info["Contains"] = "linear perturbations"
 
-    def add_non_linear_perturbations(self, NonLinearPerts, rho_background):
+    # add results from non-linear wake propagation onto the grid
+    def _add_non_linear_perturbations(self, NonLinearPerts : _NonLinearPerts, rho_background : "_Grid.rho") -> None:
+        """Add results from the non-linear regime, stored in NonLinearPerts object, to the grid
+
+        Parameters
+        ----------
+        NonLinearPerts : NonLinearPerts
+            NonLinearPerts object containing results from the non-linear regime.
+        rho_background : Grid.rho
+            unperturbed density from Grid object where make_keplerian_disk has been used.
+        """
 
         nonlin = NonLinearPerts
 
@@ -324,7 +445,7 @@ class Grid:
         # Define scale height array
         if self.info["Type"] == "cartesian":
 
-            self.get_r_phi_coords()
+            self._get_r_phi_coords()
             r = self.R_xy
             z = self.Z_xy
 
@@ -348,32 +469,35 @@ class Grid:
         # update grid info
         self.info["Contains"] = "non-linear perturbations"
 
-    def add_phantom_midplane(self, PhantomDump):
+#    def add_phantom_midplane(self, PhantomDump):
+#
+#        PD = PhantomDump
+#
+#        # Add velocities, identical at all heights
+#        ph_vr   = PD.vr  [:, np.newaxis, :]
+#        ph_vphi = PD.vphi[:, np.newaxis, :]
+#
+#        self.v_r   += ph_vr #/(1 + self.Z)
+#        self.v_phi += ph_vphi #/(1 + self.Z)
+#
+#        # Define scale height array
+#        self.H = self.p.h_ref * (self.R / self.p.r_ref)**self.p.beta
+#
+#        # Add density, scaling vertically as per thin disk assumption
+#        ph_rho   = PD.rho[:, np.newaxis, :]
+#        self.rho = ph_rho * np.exp(-0.5 * (self.Z / self.H)**2)
+#
+#        # update grid info
+#        self.info["Contains"] = "phantom mid-plane extrapolated to 3D"
 
-        PD = PhantomDump
-
-        # Add velocities, identical at all heights
-        ph_vr   = PD.vr  [:, np.newaxis, :]
-        ph_vphi = PD.vphi[:, np.newaxis, :]
-
-        self.v_r   += ph_vr #/(1 + self.Z)
-        self.v_phi += ph_vphi #/(1 + self.Z)
-
-        # Define scale height array
-        self.H = self.p.h_ref * (self.R / self.p.r_ref)**self.p.beta
-
-        # Add density, scaling vertically as per thin disk assumption
-        ph_rho   = PD.rho[:, np.newaxis, :]
-        self.rho = ph_rho * np.exp(-0.5 * (self.Z / self.H)**2)
-
-        # update grid info
-        self.info["Contains"] = "phantom mid-plane extrapolated to 3D"
-
-    def merge_grids(self, grid_to_merge):
+    # take multiple grid objects and merge them if the geometry is the same, simply by adding the components
+    def _merge_grids(self, grid_to_merge : "_Grid") -> None:
+        """Merge v_r, v_phi and rho from two compatible Grid objects (ie. with the same _Parameters)
+        """
 
         g = grid_to_merge
 
-        if type(grid_to_merge) is not Grid:
+        if type(grid_to_merge) is not _Grid:
             raise Exception("merge_grids must be given a Grid object. Merging failed.")
         
         if self.p != g.p:
@@ -387,22 +511,51 @@ class Grid:
         # update info
         self.info["Contains"] += " AND " + g.info["Contains"]
 
-    def merge_phantom_densities(self, grid_to_merge):
+#    def merge_phantom_densities(self, grid_to_merge):
+#
+#        g = grid_to_merge
+#
+#        if type(grid_to_merge) is not Grid:
+#            print("Must be given a Grid object")
+#            return False
+#        
+#        if self.p != g.p:
+#            print("Both grids must have same type parameters")
+#            return False
+#
+#        # merge data arrays
+#        self.rho = g.rho
 
-        g = grid_to_merge
+    # create plots of a constant z slice, mostly used for debugging but also shows midplane results to user
+    # if they set show_midplane_plots=True
+    def _show_disk2D(
+        self, 
+        z_slice :    int, 
+        show :      bool = False, 
+        save :      bool = False, 
+        dimless :   bool = False, 
+        vphi_lim : float = 1.0, 
+        cw :        bool = False
+    ) -> None:
+        """Generate plots of the disk at constant z value with index z_slice
 
-        if type(grid_to_merge) is not Grid:
-            print("Must be given a Grid object")
-            return False
-        
-        if self.p != g.p:
-            print("Both grids must have same type parameters")
-            return False
+        For example, mid-plane plots are generated with z_slice=0.
 
-        # merge data arrays
-        self.rho = g.rho
-
-    def show_disk2D(self, z_slice, show=False, save=False, dimless=False, vphi_lim=1.0, cw=False):   # NEED TO UPDATE SAVED DIRECTORIES FOR NEW STRUCTURE
+        Parameters
+        ----------
+        z_slice : int
+            Index in Grid of z_slice to plot.
+        show : bool, optional
+            True to show plots to user using matplotlib.pyplot.show(). Default is False.
+        save : bool, optional
+            True to save plots in directory with results. Default is False.
+        dimless: bool, optional
+            Set to True if using dimensionless results to get accurately labelled plots.
+        vphi_lim : float, optional
+            Modifier for the limits on the v_phi plot.
+        cw : bool, optional
+            True for clockwise rotation and False for anticlockwise. Since anticlockwise is default, False is default.
+        """
 
         # parameters
         savedir      = f"{self.p.system}/{self.p.name}/{self.p.m_planet}Mj"
@@ -532,7 +685,11 @@ class Grid:
         
         plt.close("all")
     
-    def write_fits_file(self):
+    # create a fits file from the results formatted specifically for use in MCFOST in combination with
+    # the .para file written by pymcfost through wakeflow
+    def _write_fits_file(self) -> None:
+        """Write a .FITS file from the results compatible with being run in MCFOST
+        """
 
         if self.info["Type"] == "cartesian":
 
@@ -594,7 +751,18 @@ class Grid:
                 f"{self.p.system}/{self.p.name}/{self.p.m_planet}Mj/mcfost.para"
             )
     
-    def save_results(self, label, printed):
+    # simply saves the grid and densities/velocities to .npy files so that they may be read later
+    def _save_results(self, label : str, printed : str) -> None:
+        """ Save results and grid to .npy files in results directory
+
+        Parameters
+        ----------
+        label : str
+            Prepended to saved file names (used in wakeflow.py to give "delta" for perturbations and "total" for perturbations 
+            applied on top of an unperturbed disk).
+        printed : str
+            Message to be printed to the user in format f"{printed} saved to {savedir}".
+        """
 
         savedir = f"{self.p.system}/{self.p.name}/{self.p.m_planet}Mj"
 
