@@ -1,84 +1,23 @@
-import yaml, sys, os
+# model_setup.py
+# Written by Thomas Hilder
+
+"""
+Contains the Parameters class responsible for storing the parameters used in a Wakeflow model. Additionally, contains
+functions used read in said parameters, check their validity, and create/replace the results directory.
+"""
+
+import yaml, os
 import shutil   as sh
 import numpy    as np
 
-def load_config_file(config_file, default_config_dict=None):
+# NOTE: contents are intended for internal use and should not be directly accessed by users
 
-    # read in config file as dictionary
-    config_dict = yaml.load(open(config_file), Loader=yaml.FullLoader)
+# class for storing physical constants, will be inherited by _Parameters class
+class _Constants:
+    """
+    Constants class for storing physical constants needed by Wakeflow. Inherited by _Parameters.
+    """
 
-    if default_config_dict is not None:
-        for key in config_dict.keys():
-            if key not in default_config_dict.keys():
-                raise Exception(f"{key} is not a valid parameter.")
-
-    return config_dict
-
-def write_config_file(config_dict, directory, filename):
-
-    with open(f'{directory}{filename}', 'w') as yaml_file:
-        yaml.dump(config_dict, yaml_file, default_flow_style=False)
-
-def run_setup(param_dict, default_param_dict=None, overwrite=False):
-
-    if default_param_dict is not None:
-        for key in param_dict.keys():
-            if key not in default_param_dict.keys():
-                raise Exception(f"{key} is not a valid parameter.")
-
-    params = Parameters(param_dict)
-
-    # do sanity checks and exit if not passed
-    params.do_sanity_checks()
-
-    # check if directory for system exists, if not create it
-    system_path = f"{params.system}/"
-    os.makedirs(system_path, exist_ok=True)
-
-    # create results directory path
-    results_path = f"{params.system}/{params.name}/"
-
-    # check if results directory already exists, ask to overwrite if yes
-    results_exist = os.path.isdir(results_path)
-    if results_exist is True:
-        if overwrite:
-            print("Overwriting previous results")
-            sh.rmtree(results_path)
-            os.makedirs(results_path)
-        else:
-            raise Exception("Results already exist for run name. Either choose a different name, or run with overwrite=True")
-
-    # create individual directories for each planet mass result
-    if params.m_planet_array is not None:
-        for mass in params.m_planet_array:
-            individual_result_path = f"{params.system}/{params.name}/{mass}Mj"
-            os.makedirs(individual_result_path, exist_ok=True)
-    else:
-        individual_result_path = f"{params.system}/{params.name}/{params.m_planet}Mj"
-        os.makedirs(individual_result_path, exist_ok=True)
-
-    # write parameters used to a file in the results directory
-    write_config_file(param_dict, results_path, f"{params.name}_config.yaml")
-
-    # run mcfost grid setup if needed
-    if params.run_mcfost or params.grid_type == "mcfost":
-
-        from .mcfost_interface import make_mcfost_parameter_file, make_mcfost_grid_data
-
-        # make directory for mcfost outputs
-        mcfost_path = f"{params.system}/{params.name}/mcfost/"
-        os.makedirs(mcfost_path, exist_ok=True)
-
-        # generate mcfost parameter file
-        make_mcfost_parameter_file(params)
-
-        # generate mcfost grid data to run analytics on
-        make_mcfost_grid_data(params)
-
-    return params
-
-
-class Constants:
     def __init__(self):
 
         from astropy import constants as c
@@ -89,18 +28,20 @@ class Constants:
         self.G_const    = c.G.cgs.value
         self.au         = c.au.cgs.value
 
+# class for storing parameters to be used in a Wakeflow model
+class _Parameters(_Constants):
+    """
+    Class for storing Wakeflow model parameters to be easily handed around by various parts of Wakeflow.
+    """
 
-class Parameters(Constants):
-    def __init__(self, config):
-
-        #print(f"Reading parameters from {config_file} \n ")
+    # read in parameters from dictionary to class attributes, calculate some needed quantities
+    def __init__(self, config: dict) -> None:
+        """Inherit constants, read in parameters to dictionary, calculate quantities like thermal mass and scale height.
+        """
 
         # inherit constants
         super().__init__()
 
-        # read in config file
-        #config = yaml.load(open(config_file), Loader=yaml.FullLoader)
-
         # run_info parameters
         self.name   = str(config["name"])
         self.system = str(config["system"])
@@ -108,6 +49,7 @@ class Parameters(Constants):
         # disk parameters
         self.m_star = float(config["m_star"])
 
+        # user can provide either single planet mass or list of planet masses
         try:
             self.m_planet       = float(config["m_planet"])
             self.m_planet_array = None
@@ -118,10 +60,14 @@ class Parameters(Constants):
         self.r_outer  = float(config["r_outer"])
         self.r_inner  = float(config["r_inner"])
         self.r_planet = float(config["r_planet"])
-        self.r_ref    = float(config["r_ref"])
+        try:
+            self.r_ref = float(config["r_ref"])
+        except:
+            self.r_ref = self.r_planet
         self.r_c      = float(config["r_c"])
         self.q        = float(config["q"])
-        self.p        = float(config["p"])
+        self.dens_p   = float(config["p"])
+        self.p        = self.dens_p + self.q - 1.5
         self.hr       = float(config["hr"])
         self.rho_ref  = float(config["dens_ref"])
 
@@ -135,7 +81,7 @@ class Parameters(Constants):
         self.PAp         = float(config["PAp"])
 
         # grid parameters
-        self.grid_type = str (config["type"])
+        self.grid_type = str (config["grid_type"])
         self.n_x       = int (config["n_x"])
         self.n_y       = int (config["n_y"])
         self.n_r       = int (config["n_r"])
@@ -163,14 +109,6 @@ class Parameters(Constants):
         self.v_max      = float(config["v_max"])
         self.n_v        = int  (config["n_v"])
 
-        # pymcfost parameters
-        #self.pymcfost_plots = bool(config["pymcfost"]["pymcfost_plots"])
-        #self.velocity_channels = list(config["pymcfost"]["velocity_channels"])
-        #self.beam = float(config["pymcfost"]["beam"])
-        #self.obs_dir = str(config["pymcfost"]["obs_dir"])
-        #self.sim_dir = str(config["pymcfost"]["sim_dir"])
-        #self.v_system = float(config["pymcfost"]["v_system"])
-
         # physical parameters
         self.gamma  = float(config["adiabatic_index"])
         self.malpha = float(config["damping_malpha"])
@@ -217,9 +155,12 @@ class Parameters(Constants):
             self.U_time = np.sqrt(self.U_len**3 / (self.G_const * self.U_mass)) 
             self.U_vel  = self.U_len / self.U_time / 1E5
 
-    def do_sanity_checks(self):
+    # checks that the combination of parameters the user has provided is sensical
+    def _do_sanity_checks(self) -> bool:
+        """Called to check that the parameters specified by the user will not break the results/run. Some parameter combinations are incompatible.
+        """
 
-        print("\n* Performing sanity checks on model parameters:")
+        print("\n* Performing checks on model parameters:")
 
         if self.m_planet != None:
             print(f"M_thermal = {self.m_thermal:.3f} M_Jup")
@@ -232,11 +173,6 @@ class Parameters(Constants):
         else:
             print(f"M_thermal = {self.m_thermal:.3f} M_Jup")
             print(f"M_planets = {np.array2string(np.array(self.m_planet_array)/self.m_thermal, precision=3, floatmode='fixed')} M_th")
-            
-            
-            #print(f"Planet masses are: ", end='')
-            #print(np.array(self.m_planet_array)/self.m_thermal, end='')
-            #print(" thermal masses")
 
             # check that planet mass does not exceed thermal mass
             exceed = False
@@ -279,3 +215,85 @@ class Parameters(Constants):
 
         print("Parameters Ok - continuing")
         return True
+
+# read in .yaml file, check keys correspond to parameters and return dictionary of parameters
+def _load_config_file(config_file: str, default_config_dict: dict = None) -> dict:
+    """Reads .yaml parameter file into a dictionary so that Wakeflow may parse it.
+    """
+
+    # read in config file as dictionary
+    config_dict = yaml.load(open(config_file), Loader=yaml.FullLoader)
+
+    # check that keys in config_dict correspond to actual Wakeflow parameters
+    if default_config_dict is not None:
+        for key in config_dict.keys():
+            if key not in default_config_dict.keys():
+                raise Exception(f"{key} is not a valid parameter.")
+
+    return config_dict
+
+# write dict to .yaml file
+def _write_config_file(config_dict: dict, directory: str, filename: str) -> None:
+    """Writes parameters dictionary to .yaml file.
+    """
+
+    with open(f'{directory}{filename}', 'w') as yaml_file:
+        yaml.dump(config_dict, yaml_file, default_flow_style=False)
+
+def _run_setup(param_dict: dict, overwrite: bool = False) -> _Parameters:
+    """Perform setup by generating parameters object, checking the parameters are okay, creating results directory,
+    writing .yaml file with parameters used to results dir, and calling MCFOST to generate grid data if needed.
+    """
+
+    params = _Parameters(param_dict)
+
+    # do sanity checks and exit if not passed
+    params._do_sanity_checks()
+
+    # check if directory for system exists, if not create it
+    system_path = f"{params.system}/"
+    os.makedirs(system_path, exist_ok=True)
+
+    # create results directory path
+    results_path = f"{params.system}/{params.name}/"
+
+    # check if results directory already exists, ask to overwrite if yes
+    results_exist = os.path.isdir(results_path)
+    if results_exist is True:
+        if overwrite:
+            print("Overwriting previous results")
+            sh.rmtree(results_path)
+            os.makedirs(results_path)
+        else:
+            raise Exception("Results already exist for run name. Either choose a different name, or run with overwrite=True")
+
+    # create individual directories for each planet mass result
+    if params.m_planet_array is not None:
+        for mass in params.m_planet_array:
+            individual_result_path = f"{params.system}/{params.name}/{mass}Mj"
+            os.makedirs(individual_result_path, exist_ok=True)
+    else:
+        individual_result_path = f"{params.system}/{params.name}/{params.m_planet}Mj"
+        os.makedirs(individual_result_path, exist_ok=True)
+
+    # write parameters used to a file in the results directory
+    _write_config_file(param_dict, results_path, f"{params.name}_config.yaml")
+
+    # run mcfost grid setup if needed
+    if params.run_mcfost or params.grid_type == "mcfost":
+
+        from .mcfost_interface import _make_mcfost_parameter_file, _make_mcfost_grid_data
+
+        # make directory for mcfost outputs
+        mcfost_path = f"{params.system}/{params.name}/mcfost/"
+        os.makedirs(mcfost_path, exist_ok=True)
+
+        # generate mcfost parameter file
+        _make_mcfost_parameter_file(params)
+
+        # generate mcfost grid data to run analytics on
+        _make_mcfost_grid_data(params)
+
+    return params
+
+
