@@ -1,14 +1,40 @@
+# non_linear_perts.py
+# Written by Thomas Hilder, Daniele Fasano and Francesco Bollati
+
+"""
+Contains the NonLinearPerts class responsible for handling the non-linear regime of the models.
+"""
+
+# type hinting without circular imports
+from __future__             import annotations
+from typing                 import TYPE_CHECKING
+
 import time
 import numpy                    as np
 import matplotlib.pyplot        as plt
 from scipy.interpolate      import griddata
 from copy                   import copy
 from tqdm                   import tqdm
-from .burgers               import solve_burgers
-from .transformations       import Eta, t, Lambda_fu, Lambda_fv, get_chi, get_dens_vel, plot_r_t
+from .burgers               import _solve_burgers
+from .transformations       import _Eta, _t, _Lambda_fu, _Lambda_fv, _get_chi, _get_dens_vel, _plot_r_t
 
-class NonLinearPerts():
-    def __init__(self, parameters, Grid):
+if TYPE_CHECKING:
+    from .model_setup       import _Parameters
+    from .grid              import _Grid
+    from .linear_perts      import _LinearPerts
+
+# NOTE: contents are intended for internal use and should not be directly accessed by users
+
+# class for non-linear regime results
+class _NonLinearPerts():
+    """
+    Class to store the results from the non-linear wake propagation away from the planet.
+    """
+
+    # intialise object, give params and grid
+    def __init__(self, parameters: _Parameters, Grid: _Grid) -> None:
+        """Setup _NonLinearPerts object by grabbing _Parameters and _Grid objects.
+        """
         
         # grab parameters object
         self.p = parameters
@@ -16,9 +42,10 @@ class NonLinearPerts():
         # should be handed an empty grid with the correct dimensions and grid setup used in the run
         self.g = Grid
 
-    def extract_ICs(self, LinearPerts, LinearPerts2=None):
-
-        #print('  * Extracting Burgers initial conditions from linear density perturbation ...')
+    # get initial conditions for burgers eqn
+    def _extract_ICs(self, LinearPerts: _LinearPerts) -> None:
+        """Extract the initial conditions for the non-linear evolution from the linear regime.
+        """
         
         # grab linear perturbations object
         lp = LinearPerts
@@ -79,12 +106,12 @@ class NonLinearPerts():
         for i in range(len(y_rest)):
 
             # eta for outer and inner
-            eta_IC_outer[i] = Eta(r_IC_outer[i], phi_IC_outer[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
-            eta_IC_inner[i] = Eta(r_IC_inner[i], phi_IC_inner[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
+            eta_IC_outer[i] = _Eta(r_IC_outer[i], phi_IC_outer[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
+            eta_IC_inner[i] = _Eta(r_IC_inner[i], phi_IC_inner[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
 
             # t for outer and inner
-            t_IC_outer[i] = t(r_IC_outer[i], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
-            t_IC_inner[i] = t(r_IC_inner[i], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
+            t_IC_outer[i] = _t(r_IC_outer[i], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
+            t_IC_inner[i] = _t(r_IC_inner[i], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
 
         # restrict eta range using eta_max
         self.eta_outer = eta_IC_outer[(eta_IC_outer > -eta_max) & (eta_IC_outer < eta_max)]
@@ -102,7 +129,7 @@ class NonLinearPerts():
             y_cut = y[(y - y_match > -eta_max) & (y - y_match < eta_max)]
             self.profile = profile[(y - y_match > -eta_max) & (y - y_match < eta_max)]
             self.eta = y_cut - y_match*np.ones(len(y_cut))
-            self.t0 = t(self.p.r_planet + self.p.l * lp.x_box, self.p.r_planet, self.p.hr, self.p.q, self.p.p)
+            self.t0 = _t(self.p.r_planet + self.p.l * lp.x_box, self.p.r_planet, self.p.hr, self.p.q, self.p.p)
 
         # set eta_tilde for outer wake:
         for i in range(len(self.eta_outer)):
@@ -152,11 +179,13 @@ class NonLinearPerts():
 
         if self.p.show_teta_debug_plots:
 
+            index = index_outer
+
             # run square box cut
-            lp.cut_box_square()
+            lp._cut_box_square()
 
             # plot r, t
-            plot_r_t(self.p)
+            _plot_r_t(self.p)
 
             # Local Cartesian grid which the linear solution was calculated on, meshgrid version
             X, Y = np.meshgrid(lp.x_cut,lp.y_cut)
@@ -190,8 +219,8 @@ class NonLinearPerts():
             linear_profile_v_r   = v_unit * (lp.cut_v_r[:,x_len:index]   / np.sqrt(np.abs(X_rest))) ###
             linear_profile_v_phi = v_unit * (lp.cut_v_phi[:,x_len:index] / np.sqrt(np.abs(X_rest))) ###
             R0  = np.sqrt((Rp + l*X_rest[:,-1])**2 + (l*Y_rest[:,-1])**2)
-            Lfu = Lambda_fu(R0, Rp, csp, hr, gamma, q, p)
-            Lfv = Lambda_fv(R0, Rp, csp, hr, gamma, q, p)
+            Lfu = _Lambda_fu(R0, Rp, csp, hr, gamma, q, p)
+            Lfv = _Lambda_fv(R0, Rp, csp, hr, gamma, q, p)
             chi0_v_r = linear_profile_v_r[:,-1] / (np.sign(R0 - Rp) * Lfu)
             chi0_v_phi = linear_profile_v_phi[:,-1] / (np.sign(R0 - Rp) * Lfv * (-cw))
 
@@ -210,16 +239,16 @@ class NonLinearPerts():
             for i in range(eta_lin.shape[0]):
                 #print(str(i))
                 for j in range(eta_lin.shape[1]):
-                    eta_lin[i,j] = Eta(r_glob[i,j], phi_glob[i,j], self.p.r_planet, self.p.hr_planet, self.p.q, -self.p.a_cw)
-                    t_lin[i,j] = t(r_glob[i,j], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
+                    eta_lin[i,j] = _Eta(r_glob[i,j], phi_glob[i,j], self.p.r_planet, self.p.hr_planet, self.p.q, -self.p.a_cw)
+                    t_lin[i,j] = _t(r_glob[i,j], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
 
             # Plot Chi vs eta when using eta transformation used for IC cut out
-            """
-            for i in range(0, len(linear_eta[0,:]), 10):
-                plt.plot(linear_eta[:,i], linear_profile[:,i])
-                plt.plot(eta_lin[:,i], linear_profile[:,i])
-                plt.show()
-            """
+
+            #for i in range(0, len(linear_eta[0,:]), 10):
+            #    plt.plot(linear_eta[:,i], linear_profile[:,i])
+            #    plt.plot(eta_lin[:,i], linear_profile[:,i])
+            #    plt.show()
+
             plt.plot(eta_lin[:,-1], chi0_rho, label="chi rho")
             plt.plot(eta_lin[:,-1], chi0_v_r, label="chi vr")
             plt.plot(eta_lin[:,-1], chi0_v_phi, label="chi vphi")
@@ -237,59 +266,6 @@ class NonLinearPerts():
                 np.save(f"{name}_eta.npy", eta_save)
                 np.save(f"{name}_chi.npy", chi_save)
 
-            if LinearPerts2 is not None: # for plotting analytics vs Phantom ICs
-
-                # grab linear perturbations object
-                lpa = LinearPerts2
-
-                # grid
-                x = lpa.X[0,:]
-                y = lpa.Y[:,0]
-
-                # set maximum eta - semi-width of the support of the azimuthal profile of the linear density perturbation
-                eta_max = 25 
-
-                # find the index in the x grid corresponding to the edge of the box
-                index = np.argmin(np.abs(x - lpa.x_box))
-
-                # extract profile of constant x along edge of box for initial condition
-                profile = lpa.pert_rho[:,index] / np.sqrt(np.abs(lpa.x_box))
-
-                # restrict y range --- I'm not sure why this is necessary but it is. Larger values will cause issues
-                y_max = 30
-                profile_rest = profile[(y > -y_max) & (y < y_max)]
-                y_rest = y[(y > -y_max) & (y < y_max)]
-
-                # ## find eta points for IC profile using full transformation
-                # find local cart. coords in real units
-                x_IC = self.p.l * np.repeat(lpa.x_box, len(y_rest))
-                y_IC = self.p.l * y_rest
-
-                # find corresponding global polar coords
-                r_IC = x_IC + self.p.r_planet
-                phi_IC = y_IC / self.p.r_planet
-
-                # initialise arrays for corresponding t,eta points
-                eta_IC = np.zeros(len(y_rest))
-                t_IC = np.zeros(len(y_rest))
-
-                # perform transformation
-                for i in range(len(y_rest)):
-                    eta_IC[i] = Eta(r_IC[i], phi_IC[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
-                    t_IC[i] = t(r_IC[i], self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
-
-                # restrict eta range using eta_max
-                self.eta2 = eta_IC[(eta_IC > -eta_max) & (eta_IC < eta_max)]
-                self.profile2 = profile_rest[(eta_IC > -eta_max) & (eta_IC < eta_max)]
-
-                plt.plot(self.eta, self.profile, ls='--', label="simulation")
-                plt.plot(self.eta2, self.profile2, ls='--', label="analytics")
-                plt.legend(loc="lower left")
-                plt.xlim(-10,10)
-                plt.xlabel('$\eta$')
-                plt.ylabel('$\chi$')
-                plt.show()
-
             plt.plot(linear_eta[:,-1], linear_profile[:,-1], label="Approximate $\eta$ transformation")
             plt.plot(eta_lin[:,-1], linear_profile[:,-1], label="Full $\eta$ transformation")
             plt.plot(self.eta, self.profile, ls='--', label="Actual IC used")
@@ -341,9 +317,12 @@ class NonLinearPerts():
             self.linear_solution = 0
             self.linear_t = 0
 
-    def extract_ICs_ann(self, LinearPerts):
-        
-        #print('  * Extracting Burgers initial conditions from linear density perturbation ...')
+    # alternative IC extraction using edge of box
+    def _extract_ICs_ann(self, LinearPerts: _LinearPerts) -> None:
+        """Alternate initial condition extraction where the IC is read from the edges of the box as included in the final
+        solution. Usually, far more y-extent of the linear regime is used than this. Using this method will invalidate the
+        solution and is meant for developer use only.
+        """
         
         # grab linear perturbations object
         lp = LinearPerts
@@ -371,8 +350,8 @@ class NonLinearPerts():
          plt.show()
 
         # find t points
-        t_IC_outer = t(r_IC_outer, self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
-        t_IC_inner = t(r_IC_inner, self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
+        t_IC_outer = _t(r_IC_outer, self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
+        t_IC_inner = _t(r_IC_inner, self.p.r_planet, self.p.hr_planet, self.p.q, self.p.p)
 
         # initialise arrays for corresponding eta points
         self.eta_outer = np.zeros(len(phi_IC_outer))
@@ -380,8 +359,8 @@ class NonLinearPerts():
 
         # perform transformation
         for i in range(len(phi_IC_outer)):
-            self.eta_outer[i] = Eta(r_IC_outer, phi_IC_outer[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
-            self.eta_inner[i] = Eta(r_IC_inner, phi_IC_inner[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
+            self.eta_outer[i] = _Eta(r_IC_outer, phi_IC_outer[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
+            self.eta_inner[i] = _Eta(r_IC_inner, phi_IC_inner[i], self.p.r_planet, self.p.hr_planet, self.p.q, -1)
 
         # set t0
         self.t0_outer = t_IC_outer
@@ -428,7 +407,11 @@ class NonLinearPerts():
         self.linear_solution = 0
         self.linear_t = 0
 
-    def get_non_linear_perts(self):
+    # propagate the wake using IC
+    def _get_non_linear_perts(self) -> None:
+        """Calculate the perturbations in the non-linear regime by propagating the wake away from the planet
+        by solving Burger's eqn and then transforming the results to physical coordinates.
+        """
 
         beta_p = self.p.m_planet / self.p.m_thermal
 
@@ -436,7 +419,7 @@ class NonLinearPerts():
 
         timer_0 = time.perf_counter()
 
-        time_outer, eta_outer, solution_outer = solve_burgers(
+        time_outer, eta_outer, solution_outer = _solve_burgers(
             self.eta_outer, 
             self.profile_outer, 
             self.p.gamma, 
@@ -459,7 +442,7 @@ class NonLinearPerts():
 
         timer_0 = time.perf_counter()
 
-        time_inner, eta_inner, solution_inner = solve_burgers(
+        time_inner, eta_inner, solution_inner = _solve_burgers(
             self.eta_inner, 
             -self.profile_inner,     # this minus sign is intended
             self.p.gamma, 
@@ -523,7 +506,7 @@ class NonLinearPerts():
                     rr = np.sqrt(xx**2 + yy**2)
                     pphi = np.arctan2(yy,xx)
 
-                    Chi = get_chi(
+                    Chi = _get_chi(
                         pphi, 
                         rr, 
                         time_outer,
@@ -550,7 +533,7 @@ class NonLinearPerts():
                     )
 
                     # COMPUTE DENSITY AND VELOCITY PERTURBATIONS
-                    dnl[i,j], unl[i,j], vnl[i,j] = get_dens_vel(
+                    dnl[i,j], unl[i,j], vnl[i,j] = _get_dens_vel(
                         rr, 
                         Chi, 
                         gamma, 
@@ -578,7 +561,7 @@ class NonLinearPerts():
 
                     pphi = phi[j]
 
-                    Chi = get_chi(
+                    Chi = _get_chi(
                         pphi, 
                         rr, 
                         time_outer,
@@ -605,7 +588,7 @@ class NonLinearPerts():
                     )
 
                     # COMPUTE DENSITY AND VELOCITY PERTURBATIONS
-                    dnl[j,i], unl[j,i], vnl[j,i] = get_dens_vel(
+                    dnl[j,i], unl[j,i], vnl[j,i] = _get_dens_vel(
                         rr, 
                         Chi, 
                         gamma, 
